@@ -1,5 +1,6 @@
 /* Close a handle opened by `dlopen'.
-   Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2004
+   Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -18,15 +19,35 @@
    02111-1307 USA.  */
 
 #include <dlfcn.h>
+#include <ldsodefs.h>
 
-static void
-dlclose_doit (void *handle)
-{
-  _dl_close (handle);
-}
+#if !defined SHARED && defined IS_IN_libdl
 
 int
 dlclose (void *handle)
 {
+  return __dlclose (handle);
+}
+
+#else
+
+static void
+dlclose_doit (void *handle)
+{
+  GLRO(dl_close) (handle);
+}
+
+int
+__dlclose (void *handle)
+{
+# ifdef SHARED
+  if (__builtin_expect (_dlfcn_hook != NULL, 0))
+    return _dlfcn_hook->dlclose (handle);
+# endif
+
   return _dlerror_run (dlclose_doit, handle) ? -1 : 0;
 }
+# ifdef SHARED
+strong_alias (__dlclose, dlclose)
+# endif
+#endif
