@@ -1,22 +1,20 @@
 /* Implementation of the locale program according to POSIX 9945-2.
-   Copyright (C) 1995-1997,1999,2000,2001,2002 Free Software Foundation, Inc.
+   Copyright (C) 1995-1997, 1999-2005, 2006 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Ulrich Drepper <drepper@cygnus.com>, 1995.
 
-   The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 2.1 of the License, or (at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License version 2 as
+   published by the Free Software Foundation.
 
-   The GNU C Library is distributed in the hope that it will be useful,
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with the GNU C Library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -43,10 +41,12 @@
 
 #include "localeinfo.h"
 #include "charmap-dir.h"
+#include "../locarchive.h"
 
 extern void *xmalloc (size_t __n);
 extern char *xstrdup (const char *__str);
 
+#define ARCHIVE_NAME LOCALEDIR "/locale-archive"
 
 /* If set print the name of the category.  */
 static int show_category_name;
@@ -83,7 +83,9 @@ static const struct argp_option options[] =
 };
 
 /* Short description of program.  */
-static const char doc[] = N_("Get locale-specific information.");
+static const char doc[] = N_("Get locale-specific information.\v\
+For bug reporting instructions, please see:\n\
+<http://www.gnu.org/software/libc/bugs.html>.\n");
 
 /* Strings for arguments in help texts.  */
 static const char args_doc[] = N_("NAME\n[-a|-m]");
@@ -91,13 +93,10 @@ static const char args_doc[] = N_("NAME\n[-a|-m]");
 /* Prototype for option handler.  */
 static error_t parse_opt (int key, char *arg, struct argp_state *state);
 
-/* Function to print some extra text in the help message.  */
-static char *more_help (int key, const char *text, void *input);
-
 /* Data structure to communicate with argp functions.  */
 static struct argp argp =
 {
-  options, parse_opt, args_doc, doc, NULL, more_help
+  options, parse_opt, args_doc, doc
 };
 
 
@@ -168,7 +167,11 @@ extern void locale_special (const char *name, int show_category_name,
 			    int show_keyword_name);
 
 /* Prototypes for local functions.  */
+static void print_LC_IDENTIFICATION (void *mapped, size_t size);
+static void print_LC_CTYPE (void *mapped, size_t size);
 static void write_locales (void);
+static int nameentcmp (const void *a, const void *b);
+static int write_archive_locales (void **all_datap, char *linebuf);
 static void write_charmaps (void);
 static void show_locale_vars (void);
 static void show_info (const char *name);
@@ -185,8 +188,10 @@ main (int argc, char *argv[])
 
   /* Set locale.  Do not set LC_ALL because the other categories must
      not be affected (according to POSIX.2).  */
-  setlocale (LC_CTYPE, "");
-  setlocale (LC_MESSAGES, "");
+  if (setlocale (LC_CTYPE, "") == NULL)
+    error (0, errno, gettext ("Cannot set LC_CTYPE to default locale"));
+  if (setlocale (LC_MESSAGES, "") == NULL)
+    error (0, errno, gettext ("Cannot set LC_MESSAGES to default locale"));
 
   /* Initialize the message catalog.  */
   textdomain (PACKAGE);
@@ -197,7 +202,9 @@ main (int argc, char *argv[])
   /* `-a' requests the names of all available locales.  */
   if (do_all != 0)
     {
-      setlocale (LC_COLLATE, "");
+      if (setlocale (LC_COLLATE, "") == NULL)
+	error (0, errno,
+	       gettext ("Cannot set LC_COLLATE to default locale"));
       write_locales ();
       exit (EXIT_SUCCESS);
     }
@@ -212,7 +219,8 @@ main (int argc, char *argv[])
 
   /* Specific information about the current locale are requested.
      Change to this locale now.  */
-  setlocale (LC_ALL, "");
+  if (setlocale (LC_ALL, "") == NULL)
+    error (0, errno, gettext ("Cannot set LC_ALL to default locale"));
 
   /* If no real argument is given we have to print the contents of the
      current locale definition variables.  These are LANG and the LC_*.  */
@@ -258,21 +266,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
 }
 
 
-static char *
-more_help (int key, const char *text, void *input)
-{
-  switch (key)
-    {
-    case ARGP_KEY_HELP_EXTRA:
-      /* We print some extra information.  */
-      return xstrdup (gettext ("\
-Report bugs using the `glibcbug' script to <bugs@gnu.org>.\n"));
-    default:
-      break;
-    }
-  return (char *) text;
-}
-
 /* Print the version information.  */
 static void
 print_version (FILE *stream, struct argp_state *state)
@@ -282,7 +275,7 @@ print_version (FILE *stream, struct argp_state *state)
 Copyright (C) %s Free Software Foundation, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2002");
+"), "2006");
   fprintf (stream, gettext ("Written by %s.\n"), "Ulrich Drepper");
 }
 
@@ -327,6 +320,74 @@ select_dirs (const struct dirent *dirent)
 }
 
 
+static void
+print_LC_IDENTIFICATION (void *mapped, size_t size)
+{
+  /* Read the information from the file.  */
+  struct
+    {
+      unsigned int magic;
+      unsigned int nstrings;
+      unsigned int strindex[0];
+    } *filedata = mapped;
+
+  if (filedata->magic == LIMAGIC (LC_IDENTIFICATION)
+      && (sizeof *filedata
+	  + (filedata->nstrings
+	     * sizeof (unsigned int))
+	  <= size))
+    {
+      const char *str;
+
+#define HANDLE(idx, name) \
+  str = ((char *) mapped						      \
+	 + filedata->strindex[_NL_ITEM_INDEX (_NL_IDENTIFICATION_##idx)]);    \
+  if (*str != '\0')							      \
+    printf ("%9s | %s\n", name, str)
+      HANDLE (TITLE, "title");
+      HANDLE (SOURCE, "source");
+      HANDLE (ADDRESS, "address");
+      HANDLE (CONTACT, "contact");
+      HANDLE (EMAIL, "email");
+      HANDLE (TEL, "telephone");
+      HANDLE (FAX, "fax");
+      HANDLE (LANGUAGE, "language");
+      HANDLE (TERRITORY, "territory");
+      HANDLE (AUDIENCE, "audience");
+      HANDLE (APPLICATION, "application");
+      HANDLE (ABBREVIATION, "abbreviation");
+      HANDLE (REVISION, "revision");
+      HANDLE (DATE, "date");
+    }
+}
+
+
+static void
+print_LC_CTYPE (void *mapped, size_t size)
+{
+  struct
+    {
+      unsigned int magic;
+      unsigned int nstrings;
+      unsigned int strindex[0];
+    } *filedata = mapped;
+
+  if (filedata->magic == LIMAGIC (LC_CTYPE)
+      && (sizeof *filedata
+	  + (filedata->nstrings
+	     * sizeof (unsigned int))
+	  <= size))
+    {
+      const char *str;
+
+      str = ((char *) mapped
+	     + filedata->strindex[_NL_ITEM_INDEX (_NL_CTYPE_CODESET_NAME)]);
+      if (*str != '\0')
+	printf ("  codeset | %s\n", str);
+    }
+}
+
+
 /* Write the names of all available locales to stdout.  We have some
    sources of the information: the contents of the locale directory
    and the locale.alias file.  To avoid duplicates and print the
@@ -358,6 +419,10 @@ write_locales (void)
   memset (linebuf, '-', sizeof (linebuf) - 1);
   linebuf[sizeof (linebuf) - 1] = '\0';
 
+  /* First scan the locale archive.  */
+  if (write_archive_locales (&all_data, linebuf))
+    first_locale = 0;
+
   /* Now we can look for all files in the directory.  */
   ndirents = scandir (LOCALEDIR, &dirents, select_dirs, alphasort);
   for (cnt = 0; cnt < ndirents; ++cnt)
@@ -375,7 +440,7 @@ write_locales (void)
 
       if (stat64 (buf, &st) == 0 && S_ISREG (st.st_mode))
 	{
-	  if (verbose)
+	  if (verbose && GET (dirents[cnt]->d_name) == NULL)
 	    {
 	      /* Provide some nice output of all kinds of
 		 information.  */
@@ -396,42 +461,7 @@ write_locales (void)
 					 MAP_SHARED, fd, 0);
 		  if (mapped != MAP_FAILED)
 		    {
-		      /* Read the information from the file.  */
-		      struct
-		      {
-			unsigned int magic;
-			unsigned int nstrings;
-			unsigned int strindex[0];
-		      } *filedata = mapped;
-
-		      if (filedata->magic == LIMAGIC (LC_IDENTIFICATION)
-			  && (sizeof *filedata
-			      + (filedata->nstrings
-				 * sizeof (unsigned int))
-			      <= (size_t) st.st_size))
-			{
-			  const char *str;
-
-#define HANDLE(idx, name) \
-  str = ((char *) mapped						      \
-	 + filedata->strindex[_NL_ITEM_INDEX (_NL_IDENTIFICATION_##idx)]);    \
-  if (*str != '\0')							      \
-    printf ("%9s | %s\n", name, str)
-			  HANDLE (TITLE, "title");
-			  HANDLE (SOURCE, "source");
-			  HANDLE (ADDRESS, "address");
-			  HANDLE (CONTACT, "contact");
-			  HANDLE (EMAIL, "email");
-			  HANDLE (TEL, "telephone");
-			  HANDLE (FAX, "fax");
-			  HANDLE (LANGUAGE, "language");
-			  HANDLE (TERRITORY, "territory");
-			  HANDLE (AUDIENCE, "audience");
-			  HANDLE (APPLICATION, "application");
-			  HANDLE (ABBREVIATION, "abbreviation");
-			  HANDLE (REVISION, "revision");
-			  HANDLE (DATE, "date");
-			}
+		      print_LC_IDENTIFICATION (mapped, st.st_size);
 
 		      munmap (mapped, st.st_size);
 		    }
@@ -446,26 +476,7 @@ write_locales (void)
 					    MAP_SHARED, fd, 0))
 			  != MAP_FAILED))
 		    {
-		      struct
-		      {
-			unsigned int magic;
-			unsigned int nstrings;
-			unsigned int strindex[0];
-		      } *filedata = mapped;
-
-		      if (filedata->magic == LIMAGIC (LC_CTYPE)
-			  && (sizeof *filedata
-			      + (filedata->nstrings
-				 * sizeof (unsigned int))
-			      <= (size_t) st.st_size))
-			{
-			  const char *str;
-
-			  str = ((char *) mapped
-				 + filedata->strindex[_NL_ITEM_INDEX (_NL_CTYPE_CODESET_NAME)]);
-			  if (*str != '\0')
-			    printf ("  codeset | %s\n", str);
-			}
+		      print_LC_CTYPE (mapped, st.st_size);
 
 		      munmap (mapped, st.st_size);
 		    }
@@ -474,10 +485,10 @@ write_locales (void)
 		    close (fd);
 		}
 	    }
-	  else
-	    /* If the verbose format is not selected we simply
-	       collect the names.  */
-	    PUT (xstrdup (dirents[cnt]->d_name));
+
+	  /* If the verbose format is not selected we simply
+	     collect the names.  */
+	  PUT (xstrdup (dirents[cnt]->d_name));
 	}
     }
   if (ndirents > 0)
@@ -495,7 +506,7 @@ write_locales (void)
       char full_name[strlen (entry) + sizeof aliasfile];
 
       stpcpy (stpcpy (full_name, entry), aliasfile);
-      fp = fopen (full_name, "r");
+      fp = fopen (full_name, "rm");
       if (fp == NULL)
 	/* Ignore non-existing files.  */
 	continue;
@@ -584,6 +595,113 @@ write_locales (void)
 }
 
 
+struct nameent
+{
+  char *name;
+  uint32_t locrec_offset;
+};
+
+
+static int
+nameentcmp (const void *a, const void *b)
+{
+  return strcoll (((const struct nameent *) a)->name,
+		  ((const struct nameent *) b)->name);
+}
+
+
+static int
+write_archive_locales (void **all_datap, char *linebuf)
+{
+  struct stat64 st;
+  void *all_data = *all_datap;
+  size_t len = 0;
+  struct locarhead *head;
+  struct namehashent *namehashtab;
+  char *addr = MAP_FAILED;
+  int fd, ret = 0;
+  uint32_t cnt;
+
+  fd = open64 (ARCHIVE_NAME, O_RDONLY);
+  if (fd < 0)
+    return 0;
+
+  if (fstat64 (fd, &st) < 0 || st.st_size < sizeof (*head))
+    goto error_out;
+
+  len = st.st_size;
+  addr = mmap64 (NULL, len, PROT_READ, MAP_SHARED, fd, 0);
+  if (addr == MAP_FAILED)
+    goto error_out;
+
+  head = (struct locarhead *) addr;
+  if (head->namehash_offset + head->namehash_size > len
+      || head->string_offset + head->string_size > len
+      || head->locrectab_offset + head->locrectab_size > len
+      || head->sumhash_offset + head->sumhash_size > len)
+    goto error_out;
+
+  namehashtab = (struct namehashent *) (addr + head->namehash_offset);
+  if (! verbose)
+    {
+      for (cnt = 0; cnt < head->namehash_size; ++cnt)
+	if (namehashtab[cnt].locrec_offset != 0)
+	  {
+	    PUT (xstrdup (addr + namehashtab[cnt].name_offset));
+	    ++ret;
+	  }
+    }
+  else
+    {
+      struct nameent *names;
+      uint32_t used;
+
+      names = (struct nameent *) xmalloc (head->namehash_used
+					  * sizeof (struct nameent));
+      for (cnt = used = 0; cnt < head->namehash_size; ++cnt)
+	if (namehashtab[cnt].locrec_offset != 0)
+	  {
+	    names[used].name = addr + namehashtab[cnt].name_offset;
+	    names[used++].locrec_offset = namehashtab[cnt].locrec_offset;
+	  }
+
+      /* Sort the names.  */
+      qsort (names, used, sizeof (struct nameent), nameentcmp);
+
+      for (cnt = 0; cnt < used; ++cnt)
+	{
+	  struct locrecent *locrec;
+
+	  PUT (xstrdup (names[cnt].name));
+
+	  if (cnt)
+	    putchar_unlocked ('\n');
+
+	  printf ("locale: %-15.15s archive: " ARCHIVE_NAME "\n%s\n",
+		  names[cnt].name, linebuf);
+
+	  locrec = (struct locrecent *) (addr + names[cnt].locrec_offset);
+
+	  print_LC_IDENTIFICATION (addr
+				   + locrec->record[LC_IDENTIFICATION].offset,
+				   locrec->record[LC_IDENTIFICATION].len);
+
+	  print_LC_CTYPE (addr + locrec->record[LC_CTYPE].offset,
+			  locrec->record[LC_CTYPE].len);
+	}
+
+      ret = used;
+    }
+
+error_out:
+  if (addr != MAP_FAILED)
+    munmap (addr, len);
+  close (fd);
+  *all_datap = all_data;
+  return ret;
+}
+
+
 /* Write the names of all available character maps to stdout.  */
 static void
 write_charmaps (void)
@@ -633,7 +751,7 @@ show_locale_vars (void)
 {
   size_t cat_no;
   const char *lcall = getenv ("LC_ALL");
-  const char *lang = getenv ("LANG") ? : "POSIX";
+  const char *lang = getenv ("LANG") ? : "";
 
   auto void get_source (const char *name);
 
@@ -642,7 +760,8 @@ show_locale_vars (void)
       char *val = getenv (name);
 
       if ((lcall ?: "")[0] != '\0' || val == NULL)
-	printf ("%s=\"%s\"\n", name, (lcall ?: "")[0] ? lcall : lang);
+	printf ("%s=\"%s\"\n", name,
+		(lcall ?: "")[0] ? lcall : (lang ?: "")[0] ? lang : "POSIX");
       else
 	printf ("%s=%s\n", name, val);
     }
@@ -733,7 +852,7 @@ show_info (const char *name)
 	      printf ("%s=", item->name);
 
 	    if (val != NULL)
-	      printf ("%d", *val == CHAR_MAX ? -1 : *val);
+	      printf ("%d", *val == '\177' ? -1 : *val);
 	    putchar ('\n');
 	  }
 	  break;
@@ -747,22 +866,22 @@ show_info (const char *name)
 
 	    while (cnt > 1)
 	      {
-		printf ("%d;", *val == CHAR_MAX ? -1 : *val);
+		printf ("%d;", *val == '\177' ? -1 : *val);
                 --cnt;
 		++val;
 	      }
 
-	    printf ("%d\n", cnt == 0 || *val == CHAR_MAX ? -1 : *val);
+	    printf ("%d\n", cnt == 0 || *val == '\177' ? -1 : *val);
 	  }
 	  break;
 	case word:
 	  {
-	    unsigned int val =
-	      (unsigned int) (unsigned long int) nl_langinfo (item->item_id);
+	    union { unsigned int word; char *string; } val;
+	    val.string = nl_langinfo (item->item_id);
 	    if (show_keyword_name)
 	      printf ("%s=", item->name);
 
-	    printf ("%d\n", val);
+	    printf ("%d\n", val.word);
 	  }
 	  break;
 	case wstring:
