@@ -1,21 +1,21 @@
 /* Miscellaneous tests which don't fit anywhere else.
-   Copyright (C) 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2004, 2005 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
    The GNU C Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with the GNU C Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with the GNU C Library; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   02111-1307 USA.  */
 
 #include <fenv.h>
 #include <float.h>
@@ -54,6 +54,10 @@ main (void)
 
 #  if LDBL_MANT_DIG == 64
     m = 0xf.fffffffffffffffp-4L;
+#  elif LDBL_MANT_DIG == 106
+    /* This has to match the mantissa of LDBL_MAX which actually does have a
+       missing bit in the middle.  */
+    m = 0x1.fffffffffffff7ffffffffffff8p-1L;
 #  elif LDBL_MANT_DIG == 113
     m = 0x1.ffffffffffffffffffffffffffffp-1L;
 #  else
@@ -79,6 +83,28 @@ main (void)
 	  }
 	puts ("ok");
       }
+
+    for (i = LDBL_MIN_EXP, x = LDBL_MIN; i >= LDBL_MIN_EXP - LDBL_MANT_DIG + 1;
+	 --i, x /= 2.0L)
+      {
+        printf ("2^%d: ", i);
+
+        r = frexpl (x, &e);
+        if (r != 0.5L)
+          {
+            printf ("mantissa incorrect: %.20La\n", r);
+            result = 1;
+            continue;
+          }
+        if (e != i)
+          {
+            printf ("exponent wrong %d (%.20Lg)\n", e, x);
+            result = 1;
+            continue;
+          }
+        puts ("ok");
+      }
+
   }
 # endif
 
@@ -362,6 +388,24 @@ main (void)
 		v2.ieee.negative);
 	result = 1;
       }
+
+    if (nextafterf (0.0f, INFINITY) != nextafterf (0.0f, 1.0f)
+        || nextafterf (-0.0f, INFINITY) != nextafterf (-0.0f, 1.0f)
+        || nextafterf (0.0f, -INFINITY) != nextafterf (0.0f, -1.0f)
+        || nextafterf (-0.0f, -INFINITY) != nextafterf (-0.0f, -1.0f))
+      {
+	printf ("nextafterf (+-0, +-Inf) != nextafterf (+-0, +-1)\n");
+	result = 1;
+      }
+
+    if (nexttowardf (0.0f, INFINITY) != nexttowardf (0.0f, 1.0f)
+        || nexttowardf (-0.0f, INFINITY) != nexttowardf (-0.0f, 1.0f)
+        || nexttowardf (0.0f, -INFINITY) != nexttowardf (0.0f, -1.0f)
+        || nexttowardf (-0.0f, -INFINITY) != nexttowardf (-0.0f, -1.0f))
+      {
+	printf ("nexttowardf (+-0, +-Inf) != nexttowardf (+-0, +-1)\n");
+	result = 1;
+      }
   }
 
   {
@@ -658,6 +702,24 @@ main (void)
 		v2.ieee.negative);
 	result = 1;
       }
+
+    if (nextafter (0.0, INFINITY) != nextafter (0.0, 1.0)
+        || nextafter (-0.0, INFINITY) != nextafter (-0.0, 1.0)
+        || nextafter (0.0, -INFINITY) != nextafter (0.0, -1.0)
+        || nextafter (-0.0, -INFINITY) != nextafter (-0.0, -1.0))
+      {
+	printf ("nextafter (+-0, +-Inf) != nextafter (+-0, +-1)\n");
+	result = 1;
+      }
+
+    if (nexttoward (0.0, INFINITY) != nexttoward (0.0, 1.0)
+        || nexttoward (-0.0, INFINITY) != nexttoward (-0.0, 1.0)
+        || nexttoward (0.0, -INFINITY) != nexttoward (0.0, -1.0)
+        || nexttoward (-0.0, -INFINITY) != nexttoward (-0.0, -1.0))
+      {
+	printf ("nexttoward (+-0, +-Inf) != nexttoward (+-0, +-1)\n");
+	result = 1;
+      }
   }
 
 #ifndef NO_LONG_DOUBLE
@@ -958,6 +1020,24 @@ main (void)
 		v2.ieee.negative);
 	result = 1;
       }
+
+    if (nextafterl (0.0, INFINITY) != nextafterl (0.0, 1.0)
+        || nextafterl (-0.0, INFINITY) != nextafterl (-0.0, 1.0)
+        || nextafterl (0.0, -INFINITY) != nextafterl (0.0, -1.0)
+        || nextafterl (-0.0, -INFINITY) != nextafterl (-0.0, -1.0))
+      {
+	printf ("nextafterl (+-0, +-Inf) != nextafterl (+-0, +-1)\n");
+	result = 1;
+      }
+
+    if (nexttowardl (0.0L, INFINITY) != nexttowardl (0.0L, 1.0L)
+        || nexttowardl (-0.0L, INFINITY) != nexttowardl (-0.0L, 1.0L)
+        || nexttowardl (0.0L, -INFINITY) != nexttowardl (0.0L, -1.0L)
+        || nexttowardl (-0.0L, -INFINITY) != nexttowardl (-0.0L, -1.0L))
+      {
+	printf ("nexttowardl (+-0, +-Inf) != nexttowardl (+-0, +-1)\n");
+	result = 1;
+      }
   }
 #endif
 
@@ -1063,5 +1143,45 @@ main (void)
   }
 #endif
 
+  /* The tests here are very similar to tests earlier in this file,
+     the important difference is just that there are no intervening
+     union variables that cause some GCC versions to hide possible
+     bugs in nextafter* implementation.  */
+  if (nextafterf (nextafterf (FLT_MIN, FLT_MIN / 2.0), FLT_MIN) != FLT_MIN)
+    {
+      puts ("nextafterf FLT_MIN test failed");
+      result = 1;
+    }
+  if (nextafterf (nextafterf (-FLT_MIN, -FLT_MIN / 2.0), -FLT_MIN)
+      != -FLT_MIN)
+    {
+      puts ("nextafterf -FLT_MIN test failed");
+      result = 1;
+    }
+  if (nextafter (nextafter (DBL_MIN, DBL_MIN / 2.0), DBL_MIN) != DBL_MIN)
+    {
+      puts ("nextafter DBL_MIN test failed");
+      result = 1;
+    }
+  if (nextafter (nextafter (-DBL_MIN, -DBL_MIN / 2.0), -DBL_MIN) != -DBL_MIN)
+    {
+      puts ("nextafter -DBL_MIN test failed");
+      result = 1;
+    }
+#ifndef NO_LONG_DOUBLE
+  if (nextafterl (nextafterl (LDBL_MIN, LDBL_MIN / 2.0), LDBL_MIN)
+      != LDBL_MIN)
+    {
+      puts ("nextafterl LDBL_MIN test failed");
+      result = 1;
+    }
+  if (nextafterl (nextafterl (-LDBL_MIN, -LDBL_MIN / 2.0), -LDBL_MIN)
+      != -LDBL_MIN)
+    {
+      puts ("nextafterl -LDBL_MIN test failed");
+      result = 1;
+    }
+#endif
+
   return result;
 }
