@@ -1,5 +1,5 @@
 /* POSIX regex testsuite from IEEE 2003.2.
-   Copyright (C) 1998 Free Software Foundation, Inc.
+   Copyright (C) 1998, 2003 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Contributed by Ulrich Drepper <drepper@cygnus.com>, 1998.
 
@@ -50,6 +50,8 @@ main (int argc, char *argv[])
 		(int) strlen (tests[cnt].reg),
 		"-----------------------------------------------------");
       }
+    else if (tests[cnt].reg == NULL)
+	printf ("!!! %s\n", tests[cnt].str);
     else
       {
 	regex_t re;
@@ -117,8 +119,5 @@ main (int argc, char *argv[])
 
   printf ("\n%Zu tests, %d errors\n", cnt, errors);
 
-  /* We should return here the error status but since some tests are known
-     to fail this would only cause the libc testsuite to fail.  */
-  //return errors != 0;
-  return 0;
+  return errors != 0;
 }
