@@ -54,7 +54,7 @@ int fd;
 void
 do_prepare (int argc, char *argv[])
 {
-   char name_len;
+   size_t name_len;
 
 #define FNAME FNAME2(TRUNCATE)
 #define FNAME2(s) "/" STRINGIFY(s) "XXXXXX"
