@@ -1,5 +1,6 @@
 /* Machine-dependent ELF dynamic relocation inline functions.  SH version.
-   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005
+   Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -20,12 +21,10 @@
 #ifndef dl_machine_h
 #define dl_machine_h
 
-/* Only dummy. This doesn't work. */
-
 #define ELF_MACHINE_NAME "SH"
 
 #include <sys/param.h>
-
+#include <sysdep.h>
 #include <assert.h>
 
 /* Return nonzero iff ELF header is compatible with the running host.  */
@@ -71,7 +70,7 @@ elf_machine_load_address (void)
 /* Set up the loaded object described by L so its unrelocated PLT
    entries will jump to the on-demand fixup code in dl-runtime.c.  */
 
-static inline int __attribute__ ((unused))
+static inline int __attribute__ ((unused, always_inline))
 elf_machine_runtime_setup (struct link_map *l, int lazy, int profile)
 {
   Elf32_Addr *got;
@@ -105,7 +104,9 @@ elf_machine_runtime_setup (struct link_map *l, int lazy, int profile)
 	{
 	  got[2] = (Elf32_Addr) &_dl_runtime_profile;
 	  /* Say that we really want profiling and the timers are started.  */
-	  GL(dl_profile_map) = l;
+	  if (GLRO(dl_profile) != NULL
+	      && _dl_name_match_p (GLRO(dl_profile), l))
+	    GL(dl_profile_map) = l;
 	}
       else
 	/* This function will get called to fix up the GOT entry indicated by
@@ -115,196 +116,8 @@ elf_machine_runtime_setup (struct link_map *l, int lazy, int profile)
   return lazy;
 }
 
-/* This code is used in dl-runtime.c to call the `fixup' function
-   and then redirect to the address it returns.	 */
-
 #define ELF_MACHINE_RUNTIME_FIXUP_ARGS int plt_type
 
-#ifdef SHARED
-#define FUN_ADDR	"\
-	mov.l 1f,r2\n\
-	mova 1f,r0\n\
-        bra 2f\n\
-	 add r0,r2		! Get GOT address in r2\n\
-0:	.align 2\n\
-1:	.long _GLOBAL_OFFSET_TABLE_\n\
-2:	mov.l 3f,r0\n\
-	add r2,r0"
-#define GOTJMP(x)	#x "@GOTOFF"
-#else
-#define FUN_ADDR	"\
-	mov.l 3f,r0"
-#define GOTJMP(x)	#x
-#endif
-
-#ifdef HAVE_FPU
-#define FGR_SAVE	"\
-	sts.l	fpscr, @-r15\n\
-	mov	#8,r3\n\
-	swap.w	r3, r3\n\
-	lds	r3, fpscr\n\
-	fmov.s	fr11, @-r15\n\
-	fmov.s	fr10, @-r15\n\
-	fmov.s	fr9, @-r15\n\
-	fmov.s	fr8, @-r15\n\
-	fmov.s	fr7, @-r15\n\
-	fmov.s	fr6, @-r15\n\
-	fmov.s	fr5, @-r15\n\
-	fmov.s	fr4, @-r15"
-#define FGR_LOAD	"\
-	fmov.s	@r15+, fr4\n\
-	fmov.s	@r15+, fr5\n\
-	fmov.s	@r15+, fr6\n\
-	fmov.s	@r15+, fr7\n\
-	fmov.s	@r15+, fr8\n\
-	fmov.s	@r15+, fr9\n\
-	fmov.s	@r15+, fr10\n\
-	fmov.s	@r15+, fr11\n\
-	lds.l	@r15+, fpscr"
-#else
-#define FGR_SAVE	""
-#define FGR_LOAD	""
-#endif
-
-#ifndef PROF
-# define ELF_MACHINE_RUNTIME_TRAMPOLINE asm ("\
-	.text\n\
-	.globl _dl_runtime_resolve\n\
-	.type _dl_runtime_resolve, @function\n\
-	.align 5\n\
-_dl_runtime_resolve:\n\
-	mov.l r2,@-r15\n\
-	mov.l r3,@-r15\n\
-	mov.l r4,@-r15\n\
-	mov.l r5,@-r15\n\
-	mov.l r6,@-r15\n\
-	mov.l r7,@-r15\n\
-	mov.l r12,@-r15\n\
-	movt r3			! Save T flag.\n\
-	mov.l r3,@-r15\n\
-	" FGR_SAVE "\n\
-	sts.l pr,@-r15\n\
-	tst r0,r0\n\
-	bt 1f\n\
-	mov r0,r2\n\
-1:\n\
-	mov r0,r4		! PLT type\n\
-	mov r2,r5		! link map address\n\
-	" FUN_ADDR "\n\
-	jsr @r0			! Call resolver.\n\
-	 mov r1,r6		! reloc offset\n\
-	lds.l @r15+,pr		! Get register content back.\n\
-	" FGR_LOAD "\n\
-	mov.l @r15+,r3\n\
-	shal r3			! Lode T flag.\n\
-	mov.l @r15+,r12\n\
-	mov.l @r15+,r7\n\
-	mov.l @r15+,r6\n\
-	mov.l @r15+,r5\n\
-	mov.l @r15+,r4\n\
-	mov.l @r15+,r3\n\
-	jmp @r0			! Jump to function address.\n\
-	 mov.l @r15+,r2\n\
-	.align 2\n\
-3:\n\
-	.long " GOTJMP (fixup) "\n\
-	.size _dl_runtime_resolve, .-_dl_runtime_resolve\n\
-\n\
-	.globl _dl_runtime_profile\n\
-	.type _dl_runtime_profile, @function\n\
-	.align 5\n\
-_dl_runtime_profile:\n\
-	mov.l r2,@-r15\n\
-	mov.l r3,@-r15\n\
-	mov.l r4,@-r15\n\
-	mov.l r5,@-r15\n\
-	mov.l r6,@-r15\n\
-	mov.l r7,@-r15\n\
-	mov.l r12,@-r15\n\
-	movt r3			! Save T flag.\n\
-	mov.l r3,@-r15\n\
-	" FGR_SAVE "\n\
-	sts.l pr,@-r15\n\
-	tst r0,r0\n\
-	bt 1f\n\
-	mov r0,r2\n\
-1:\n\
-	mov r0,r4		! PLT type\n\
-	mov r2,r5		! link map address\n\
-	sts pr,r7		! return address\n\
-	" FUN_ADDR "\n\
-	jsr @r0			! Call resolver.\n\
-	 mov r1,r6		! reloc offset\n\
-	lds.l @r15+,pr		! Get register content back.\n\
-	" FGR_LOAD "\n\
-	mov.l @r15+,r3\n\
-	shal r3			! Lode T flag.\n\
-	mov.l @r15+,r12\n\
-	mov.l @r15+,r7\n\
-	mov.l @r15+,r6\n\
-	mov.l @r15+,r5\n\
-	mov.l @r15+,r4\n\
-	mov.l @r15+,r3\n\
-	jmp @r0			! Jump to function address.\n\
-	 mov.l @r15+,r2\n\
-	.align 2\n\
-3:\n\
-	.long " GOTJMP (profile_fixup) "\n\
-	.size _dl_runtime_profile, .-_dl_runtime_profile\n\
-	.previous\n\
-");
-#else
-# define ELF_MACHINE_RUNTIME_TRAMPOLINE asm ("\
-	.text\n\
-	.globl _dl_runtime_resolve\n\
-	.globl _dl_runtime_profile\n\
-	.type _dl_runtime_resolve, @function\n\
-	.type _dl_runtime_profile, @function\n\
-	.align 5\n\
-_dl_runtime_resolve:\n\
-_dl_runtime_profile:\n\
-	mov.l r2,@-r15\n\
-	mov.l r3,@-r15\n\
-	mov.l r4,@-r15\n\
-	mov.l r5,@-r15\n\
-	mov.l r6,@-r15\n\
-	mov.l r7,@-r15\n\
-	mov.l r12,@-r15\n\
-	movt r3			! Save T flag.\n\
-	mov.l r3,@-r15\n\
-	" FGR_SAVE "\n\
-	sts.l pr,@-r15\n\
-	tst r0,r0\n\
-	bt 1f\n\
-	mov r0,r2\n\
-1:\n\
-	mov r0,r4		! PLT type\n\
-	mov r2,r5		! link map address\n\
-	sts pr,r7		! return address\n\
-	" FUN_ADDR "\n\
-	jsr @r0			! Call resolver.\n\
-	 mov r1,r6		! reloc offset\n\
-	lds.l @r15+,pr		! Get register content back.\n\
-	" FGR_LOAD "\n\
-	mov.l @r15+,r3\n\
-	shal r3			! Lode T flag.\n\
-	mov.l @r15+,r12\n\
-	mov.l @r15+,r7\n\
-	mov.l @r15+,r6\n\
-	mov.l @r15+,r5\n\
-	mov.l @r15+,r4\n\
-	mov.l @r15+,r3\n\
-	jmp @r0			! Jump to function address.\n\
-	 mov.l @r15+,r2\n\
-	.align 2\n\
-3:\n\
-	.long " GOTJMP (fixup) "\n\
-	.size _dl_runtime_resolve, .-_dl_runtime_resolve\n\
-	.size _dl_runtime_profile, .-_dl_runtime_profile\n\
-	.previous\n\
-");
-#endif
-
 /* Mask identifying addresses reserved for the user program,
    where the dynamic linker should not map anything.  */
 #define ELF_MACHINE_USER_ADDRESS_MASK	0x80000000UL
@@ -354,6 +167,7 @@ _dl_start_user:\n\
 	mov r5,r7\n\
 	shll2 r7\n\
 	add r15,r7\n\
+	add #8,r7\n\
 	mov.l .L_dl_loaded,r0\n\
 	mov.l @(r0,r12),r0\n\
 	mov.l @r0,r4\n\
@@ -363,12 +177,7 @@ _dl_start_user:\n\
 	add r1,r0\n\
 	jsr @r0\n\
 	 nop\n\
-1:	! Clear the startup flag.\n\
-	mov.l .L_dl_starting_up,r0\n\
-	mov.l @(r0,r12),r0\n\
-	mov #0,r2\n\
-	mov.l r2,@r0\n\
-	! Pass our finalizer function to the user in r4, as per ELF ABI.\n\
+1:	! Pass our finalizer function to the user in r4, as per ELF ABI.\n\
 	mov.l .L_dl_fini,r0\n\
 	mov.l @(r0,r12),r4\n\
 	! Jump to the user's entry point.\n\
@@ -383,10 +192,14 @@ _dl_start_user:\n\
 	.long _dl_init_internal@PLT\n\
 .L_dl_loaded:\n\
 	.long _rtld_local@GOT\n\
-.L_dl_starting_up:\n\
-	.long _dl_starting_up@GOT\n\
 .L_dl_fini:\n\
 	.long _dl_fini@GOT\n\
+	.type __fpscr_values,@object\n\
+	.global __fpscr_values\n\
+__fpscr_values:\n\
+	.long   0\n\
+	.long   0x80000\n\
+	.weak __fpscr_values\n\
 .previous\n\
 ");
 
@@ -417,9 +230,9 @@ _dl_start_user:\n\
 static inline void __attribute__ ((unused))
 dl_platform_init (void)
 {
-  if (GL(dl_platform) != NULL && *GL(dl_platform) == '\0')
+  if (GLRO(dl_platform) != NULL && *GLRO(dl_platform) == '\0')
     /* Avoid an empty string which would disturb us.  */
-    GL(dl_platform) = NULL;
+    GLRO(dl_platform) = NULL;
 }
 
 static inline Elf32_Addr
@@ -438,17 +251,21 @@ elf_machine_plt_value (struct link_map *map, const Elf32_Rela *reloc,
   return value + reloc->r_addend;
 }
 
-#endif /* !dl_machine_h */
+#define ARCH_LA_PLTENTER sh_gnu_pltenter
+#define ARCH_LA_PLTEXIT sh_gnu_pltexit
 
-#ifdef RESOLVE
+#endif /* !dl_machine_h */
 
 /* SH never uses Elf32_Rel relocations.	 */
 #define ELF_MACHINE_NO_REL 1
 
+#ifdef RESOLVE_MAP
+
 /* Perform the relocation specified by RELOC and SYM (which is fully resolved).
    MAP is the object containing the reloc.  */
 
-static inline void
+auto inline void
+__attribute ((always_inline))
 elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
 		  const Elf32_Sym *sym, const struct r_found_version *version,
 		  void *const reloc_addr_arg)
@@ -460,20 +277,23 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
 #define COPY_UNALIGNED_WORD(swp, twp, align) \
   { \
     void *__s = (swp), *__t = (twp); \
+    unsigned char *__s1 = __s, *__t1 = __t; \
+    unsigned short *__s2 = __s, *__t2 = __t; \
+    unsigned long *__s4 = __s, *__t4 = __t; \
     switch ((align)) \
     { \
     case 0: \
-      *(unsigned long *) __t = *(unsigned long *) __s; \
+      *__t4 = *__s4; \
       break; \
     case 2: \
-      *((unsigned short *) __t)++ = *((unsigned short *) __s)++; \
-      *((unsigned short *) __t) = *((unsigned short *) __s); \
+      *__t2++ = *__s2++; \
+      *__t2 = *__s2; \
       break; \
     default: \
-      *((unsigned char *) __t)++ = *((unsigned char *) __s)++; \
-      *((unsigned char *) __t)++ = *((unsigned char *) __s)++; \
-      *((unsigned char *) __t)++ = *((unsigned char *) __s)++; \
-      *((unsigned char *) __t) = *((unsigned char *) __s); \
+      *__t1++ = *__s1++; \
+      *__t1++ = *__s1++; \
+      *__t1++ = *__s1++; \
+      *__t1 = *__s1; \
       break; \
     } \
   }
@@ -503,18 +323,9 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
   else
     {
       const Elf32_Sym *const refsym = sym;
-#if defined USE_TLS && !defined RTLD_BOOTSTRAP
       struct link_map *sym_map = RESOLVE_MAP (&sym, version, r_type);
 
-      value = sym == NULL ? 0 : sym_map->l_addr + sym->st_value;
-#else
-
-      value = RESOLVE (&sym, version, r_type);
-# ifndef RTLD_BOOTSTRAP
-      if (sym != NULL)
-# endif
-	value += sym->st_value;
-#endif
+      value = sym_map == NULL ? 0 : sym_map->l_addr + sym->st_value;
       value += reloc->r_addend;
 
       switch (r_type)
@@ -525,7 +336,7 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
 	       found.  */
 	    break;
 	  if (sym->st_size > refsym->st_size
-	      || (sym->st_size < refsym->st_size && GL(dl_verbose)))
+	      || (sym->st_size < refsym->st_size && GLRO(dl_verbose)))
 	    {
 	      const char *strtab;
 
@@ -576,10 +387,12 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
 	     It is a positive value which will be added to the thread
 	     pointer.  To get the variable position in the TLS block
 	     we add the offset from that of the TLS block.  */
-	  CHECK_STATIC_TLS (map, sym_map);
-	  *reloc_addr
-	    = ((sym == NULL ? 0 : sym_map->l_tls_offset + sym->st_value)
-	       + reloc->r_addend);
+	  if (sym != NULL)
+	    {
+	      CHECK_STATIC_TLS (map, sym_map);
+	      *reloc_addr = sym_map->l_tls_offset + sym->st_value
+			    + reloc->r_addend;
+	    }
 # endif
 	  break;
 #endif	/* use TLS */
@@ -593,7 +406,7 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
 	      rtld.c contains the common defn for _dl_rtld_map, which
 	      is incompatible with a weak decl in the same file.  */
 # ifndef SHARED
-	    weak_extern (GL(dl_rtld_map));
+	    weak_extern (_dl_rtld_map);
 # endif
 	    if (map == &GL(dl_rtld_map))
 	      /* Undo the relocation done here during bootstrapping.
@@ -619,11 +432,11 @@ elf_machine_rela (struct link_map *map, const Elf32_Rela *reloc,
     }
 }
 
-static inline void
+auto inline void
+__attribute__ ((always_inline))
 elf_machine_rela_relative (Elf32_Addr l_addr, const Elf32_Rela *reloc,
 			   void *const reloc_addr_arg)
 {
-  Elf32_Addr *const reloc_addr = reloc_addr_arg;
   Elf32_Addr value;
 
   if (reloc->r_addend)
@@ -638,7 +451,8 @@ elf_machine_rela_relative (Elf32_Addr l_addr, const Elf32_Rela *reloc,
 #undef COPY_UNALIGNED_WORD
 }
 
-static inline void
+auto inline void
+__attribute__ ((always_inline))
 elf_machine_lazy_rel (struct link_map *map,
 		      Elf32_Addr l_addr, const Elf32_Rela *reloc)
 {
@@ -657,4 +471,4 @@ elf_machine_lazy_rel (struct link_map *map,
     _dl_reloc_bad_type (map, ELF32_R_TYPE (reloc->r_info), 1);
 }
 
-#endif /* RESOLVE */
+#endif /* RESOLVE_MAP */
