@@ -1,5 +1,5 @@
 /* POSIX.1 `sigaction' call for Linux/i386.
-   Copyright (C) 1991, 95, 96, 97, 98, 99, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1991,1995-2000,2002-2004,2005 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
 
 #include <sysdep.h>
 #include <sys/syscall.h>
+#include <ldsodefs.h>
 
 #include <kernel-features.h>
 
@@ -37,19 +38,26 @@
 #define SA_RESTORER 0x04000000
 
 
-extern int __syscall_rt_sigaction (int, const struct kernel_sigaction *__unbounded,
-				   struct kernel_sigaction *__unbounded, size_t);
-
 #if __ASSUME_REALTIME_SIGNALS == 0
 /* The variable is shared between all wrappers around signal handling
    functions which have RT equivalents.  */
 int __libc_missing_rt_sigs;
 #endif
 
-#ifdef __NR_rt_sigaction
+/* Using the hidden attribute here does not change the code but it
+   helps to avoid warnings.  */
+#if defined HAVE_HIDDEN && defined HAVE_VISIBILITY_ATTRIBUTE \
+    && !defined HAVE_BROKEN_VISIBILITY_ATTRIBUTE
+# ifdef __NR_rt_sigaction
+extern void restore_rt (void) asm ("__restore_rt") attribute_hidden;
+# endif
+extern void restore (void) asm ("__restore") attribute_hidden;
+#else
+# ifdef __NR_rt_sigaction
 static void restore_rt (void) asm ("__restore_rt");
-#endif
+# endif
 static void restore (void) asm ("__restore");
+#endif
 
 
 /* If ACT is not NULL, change the action for SIG to *ACT.
@@ -77,11 +85,16 @@ __libc_sigaction (int sig, const struct sigaction *act, struct sigaction *oact)
       if (act)
 	{
 	  kact.k_sa_handler = act->sa_handler;
+	  kact.sa_flags = act->sa_flags;
 	  memcpy (&kact.sa_mask, &act->sa_mask, sizeof (sigset_t));
-	  kact.sa_flags = act->sa_flags | SA_RESTORER;
 
-	  kact.sa_restorer = ((act->sa_flags & SA_SIGINFO)
-			      ? &restore_rt : &restore);
+	  if (GLRO(dl_sysinfo_dso) == NULL)
+	    {
+	      kact.sa_flags |= SA_RESTORER;
+
+	      kact.sa_restorer = ((act->sa_flags & SA_SIGINFO)
+				  ? &restore_rt : &restore);
+	    }
 	}
 
       /* XXX The size argument hopefully will have to be changed to the
@@ -121,20 +134,12 @@ __libc_sigaction (int sig, const struct sigaction *act, struct sigaction *oact)
       k_newact.sa_restorer = &restore;
     }
 
-  asm volatile ("pushl %%ebx\n"
-		"movl %2, %%ebx\n"
-		"int $0x80\n"
-		"popl %%ebx"
-		: "=a" (result)
-		: "0" (SYS_ify (sigaction)), "r" (sig),
-		  "c" (act ? __ptrvalue (&k_newact) : 0),
-		  "d" (oact ? __ptrvalue (&k_oldact) : 0));
+  result = INLINE_SYSCALL (sigaction, 3, sig,
+			   act ? __ptrvalue (&k_newact) : 0,
+			   oact ? __ptrvalue (&k_oldact) : 0);
 
   if (result < 0)
-    {
-      __set_errno (-result);
-      return -1;
-    }
+    return -1;
 
   if (oact)
     {
@@ -147,9 +152,17 @@ __libc_sigaction (int sig, const struct sigaction *act, struct sigaction *oact)
   return 0;
 #endif
 }
+libc_hidden_def (__libc_sigaction)
 
+#ifdef WRAPPER_INCLUDE
+# include WRAPPER_INCLUDE
+#endif
+
+#ifndef LIBC_SIGACTION
 weak_alias (__libc_sigaction, __sigaction)
+libc_hidden_weak (__sigaction)
 weak_alias (__libc_sigaction, sigaction)
+#endif
 
 /* NOTE: Please think twice before making any changes to the bits of
    code below.  GDB needs some intimate knowledge about it to
@@ -176,8 +189,8 @@ RESTORE (restore_rt, __NR_rt_sigreturn)
 #endif
 
 /* For the boring old signals.  */
-# undef RESTORE2
-# define RESTORE2(name, syscall) \
+#undef RESTORE2
+#define RESTORE2(name, syscall) \
 asm						\
   (						\
    ".text\n"					\
