@@ -23,5 +23,5 @@
 static void
 profil_counter (int signo, SIGCONTEXT scp)
 {
-  profil_count((void *) ((unsigned long) GET_PC (scp) & 0x7fffffffUL));
+  profil_count ((void *) GET_PC (scp));
 }
