@@ -1,28 +1,26 @@
 /* `ptrace' debugger support interface.  Linux version.
-   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1996-1999,2000,2006 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
 
    The GNU C Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
+   Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with the GNU C Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU Lesser General Public
+   License along with the GNU C Library; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   02111-1307 USA.  */
 
 #ifndef _SYS_PTRACE_H
-
 #define _SYS_PTRACE_H	1
-#include <features.h>
 
-#include <asm/ptrace.h>
+#include <features.h>
 
 __BEGIN_DECLS
 
@@ -36,52 +34,99 @@ enum __ptrace_request
 #define PT_TRACE_ME PTRACE_TRACEME
 
   /* Return the word in the process's text space at address ADDR.  */
-  PTRACE_PEEKTEXT,
+  PTRACE_PEEKTEXT = 1,
 #define PT_READ_I PTRACE_PEEKTEXT
 
   /* Return the word in the process's data space at address ADDR.  */
-  PTRACE_PEEKDATA,
+  PTRACE_PEEKDATA = 2,
 #define PT_READ_D PTRACE_PEEKDATA
 
   /* Return the word in the process's user area at offset ADDR.  */
-  PTRACE_PEEKUSER,
+  PTRACE_PEEKUSER = 3,
 #define PT_READ_U PTRACE_PEEKUSER
 
   /* Write the word DATA into the process's text space at address ADDR.  */
-  PTRACE_POKETEXT,
+  PTRACE_POKETEXT = 4,
 #define PT_WRITE_I PTRACE_POKETEXT
 
   /* Write the word DATA into the process's data space at address ADDR.  */
-  PTRACE_POKEDATA,
+  PTRACE_POKEDATA = 5,
 #define PT_WRITE_D PTRACE_POKEDATA
 
   /* Write the word DATA into the process's user area at offset ADDR.  */
-  PTRACE_POKEUSER,
+  PTRACE_POKEUSER = 6,
 #define PT_WRITE_U PTRACE_POKEUSER
 
   /* Continue the process.  */
-  PTRACE_CONT,
+  PTRACE_CONT = 7,
 #define PT_CONTINUE PTRACE_CONT
 
   /* Kill the process.  */
-  PTRACE_KILL,
+  PTRACE_KILL = 8,
 #define PT_KILL PTRACE_KILL
 
   /* Single step the process.
      This is not supported on all machines.  */
-  PTRACE_SINGLESTEP,
+  PTRACE_SINGLESTEP = 9,
 #define PT_STEP PTRACE_SINGLESTEP
 
+  /* Get all general purpose registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_GETREGS = 12,
+#define PT_GETREGS PTRACE_GETREGS
+
+  /* Set all general purpose registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_SETREGS = 13,
+#define PT_SETREGS PTRACE_SETREGS
+
+  /* Get all floating point registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_GETFPREGS = 14,
+#define PT_GETFPREGS PTRACE_GETFPREGS
+
+  /* Set all floating point registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_SETFPREGS = 15,
+#define PT_SETFPREGS PTRACE_SETFPREGS
+
   /* Attach to a process that is already running. */
-  PTRACE_ATTACH = 0x10,
+  PTRACE_ATTACH = 16,
 #define PT_ATTACH PTRACE_ATTACH
 
   /* Detach from a process attached to with PTRACE_ATTACH.  */
-  PTRACE_DETACH,
+  PTRACE_DETACH = 17,
 #define PT_DETACH PTRACE_DETACH
 
+  /* Get all extended floating point registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_GETFPXREGS = 18,
+#define PT_GETFPXREGS PTRACE_GETFPXREGS
+
+  /* Set all extended floating point registers used by a processes.
+     This is not supported on all machines.  */
+   PTRACE_SETFPXREGS = 19,
+#define PT_SETFPXREGS PTRACE_SETFPXREGS
+
   /* Continue and stop at the next (return from) syscall.  */
   PTRACE_SYSCALL = 24,
+#define PT_SYSCALL PTRACE_SYSCALL
+
+  /* Set ptrace filter options.  */
+  PTRACE_SETOPTIONS = 0x4200,
+#define PT_SETOPTIONS PTRACE_SETOPTIONS
+
+  /* Get last ptrace message.  */
+  PTRACE_GETEVENTMSG = 0x4201,
+#define PT_GETEVENTMSG PTRACE_GETEVENTMSG
+
+  /* Get siginfo for process.  */
+  PTRACE_GETSIGINFO = 0x4202,
+#define PT_GETSIGINFO PTRACE_GETSIGINFO
+
+  /* Set new siginfo for process.  */
+  PTRACE_SETSIGINFO = 0x4203
+#define PT_SETSIGINFO PTRACE_SETSIGINFO
 };
 
 /* Perform process tracing functions.  REQUEST is one of the values
@@ -93,7 +138,7 @@ enum __ptrace_request
    appear (those that are used for the particular request) as:
      pid_t PID, void *ADDR, int DATA, void *ADDR2
    after REQUEST.  */
-extern long int ptrace __P ((enum __ptrace_request __request, ...));
+extern long int ptrace (enum __ptrace_request __request, ...) __THROW;
 
 __END_DECLS
 
